//
// File:       square.cu
//
// Abstract:   A simple compute example showing basic usage of CUDA which
//             calculates the mathematical square (X[i] = pow(X[i],2)) for a buffer of
//             floating point values.
//

////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <cuda.h>

////////////////////////////////////////////////////////////////////////////////

// Use a static data size for simplicity
//
#define DATA_SIZE (1024)

float* d_data;
float* d_results;

////////////////////////////////////////////////////////////////////////////////

// Simple compute kernel which computes the square of an input array 
//
// Device code
__global__ void square(const float* input, float* output, unsigned int count)
{
  int i = blockDim.x * blockIdx.x + threadIdx.x;
  if(i < count)
    output[i] = input[i] * input[i];
}

////////////////////////////////////////////////////////////////////////////////

// Host code
int main(int argc, char** argv)
{
    float h_data[DATA_SIZE];              // original data set given to device
    float h_results[DATA_SIZE];           // results returned from device
    unsigned int correct;               // number of correct results returned
    size_t size = DATA_SIZE * sizeof(float);

    // Fill our data set with random float values
    //
    int i = 0;
    unsigned int count = DATA_SIZE;
    for(i = 0; i < count; i++)
        h_data[i] = rand() / (float)RAND_MAX;
        
    // Allocate arrays in device memory
    //
    cudaMalloc((void**)&d_data, size);
    cudaMalloc((void**)&d_results, size);
    
    // Copy array from host memory to device memory
    //
    cudaMemcpy (d_data, h_data, size, cudaMemcpyHostToDevice);
    
    // Invoke kernel
    //
    int threadsPerBlock = 256;
    int blocksPerGrid = (DATA_SIZE + threadsPerBlock - 1) / threadsPerBlock;
    square<<<blocksPerGrid, threadsPerBlock>>>(d_data, d_results, count);
    
    // Copy result from device memory to host memory
    // h_results contains the result in host memory
    //
    cudaMemcpy(h_results, d_results, size, cudaMemcpyDeviceToHost);
    
    // Validate our results
    //
    correct = 0;
    for(i = 0; i < count; i++)
    {
        if(h_results[i] == h_data[i] * h_data[i])
            correct++;
    }
    
    // Print a brief summary detailing the results
    //
    printf("Computed '%d/%d' correct values!\n", correct, count);
    
    // Shutdown and cleanup
    //
    if (d_data)     cudaFree(d_data);
    if (d_results)  cudaFree(d_results);
    cudaThreadExit();

    return 0;
}

